
# A tcl script for backbone hydrogen bond eneryg analysis
# The script calculates the energy for selected hydrogen over the course of an trajectory
# loaded in VMD.

#Authors:  Tim Isgro, Elizabeth Villa and Mu Gao


# A data file will be generated to store h-bond energies
set datfile I27-hbond.dat

# window size for smoothing the hydrogen bond energy
set lowpassWin 5

#Get the number of frames in the dcd file
set nf [molinfo top get numframes]

# roll back to the first frame
animate goto 0

# Based on the initial structure, the program will search for backbone hydrogen
# bonds using cutoff of 3.5 A for donor/acceptor distance and 40 degrees for 
# D-H-A bond angle.  Note that these values may be modified. The search will 
# returns four lists of atoms involved in forming hydrongen bonds, namely

# AA1 A1 D1 H1 
# AA2 A2 D2 H2 ...

# where AA is the acceptor antecedent (C), D is donor (N),
# A is acceptor (O) and H is hydrogen.
# You might want to look for extra interesting candidates for
# H-bonds, i.e., intermonomer beta sheets.  Make sure to include
# them in the same order as above (AA1 A1 D1 H1).

set sel [atomselect top "name C O HN N and (within 1.5 of backbone and betasheet) and not index 794 1155"]
set at [measure hbonds 3.5 40 $sel]
$sel delete

set acc [lindex $at 1]
set hyd [lindex $at 2]
set don [lindex $at 0]

foreach i $acc {
  set foo [atomselect top "name C and within 1.5 of index $i"]
 #   puts "antec  [$foo get index] " 

  lappend aa [$foo get index]
#   echo $aa
$foo delete
}

# VMD may not find all hydrogen bonds you are interested. In this case
# you may add these hydrogen bonds manually, as in the following example:
#
# lappend aa  12 34 287
# lappend acc  13 35 288
# lappend don  8 382 109
# lappend hyd  9 384 112
#
# where the command lappend adds this atoms to the cooresponding lists.  
# Make sure that index number is correctly recognized by VMD. In VMD 
# indices start from 0 as opposed to 1 in the pdb file.


# additional A-B A'-G hbonds of titin I27

lappend aa 160 1263 1342
lappend acc 161 1264 1343
lappend don 1251 169 236
lappend hyd 1252 170 237


puts "This is the list of selected hydrogen bonds (acceptor antecedent, acceptor, hydrogen, and donor heavy atom):"

foreach h $aa i $acc j $don k $hyd { 
 puts "$h $i $j $k"
}


# We introduce here a procedure to define the H-Bond energy for each set of atoms. This 
# calculation is based on the parameters from CHARMM22 par_hbond.inp parameter file.

set E_NO -4.90
set R_NO 2.9
set AEXP 4 	
set REXP 6 	
set HAEX 2 	
set AAEX 2 	
set CUTHB 6.0 
set CTOFHB 5.0 
set CTONHB 4.0 
# divide by 57.3 to convert degrees to radians (as needed by VMD)
set CUTHA [expr 100.0/57.3]  
set CTOFHA [expr 90.0/57.3] 
set CTONHA [expr 90.0/57.3]


proc get_A {E R} {
    expr 3*$E*pow($R,4)
}

proc get_B {E R} {
    expr 2*$E*pow($R,6)
}

set A0 [get_A $E_NO $R_NO]
set B0 [get_B $E_NO $R_NO]

proc get_min { a b } {
 lindex [lsort -real -increasing [list $a $b]] 0 }

#the switching function
#fixed according to ONLINE x-plor manual
#Note, however, that even the online manual is incorrect, since the SW is negative...the version below fixes it and is positive from domain 1 to 0 over range Ron to Roff.

proc SW {R Ron Roff} {
    set min [get_min $R $Ron]
    if {$min == $R} then {return 1} else {
	set min [get_min $R $Roff]
	if {$min == $Roff} then {return 0} else {
	    return [expr ( pow(($R*$R)-($Roff*$Roff),2)*($R*$R-$Roff*$Roff-3*($R*$R-$Ron*$Ron))/pow((-$Roff*$Roff+$Ron*$Ron),3) )]
	}
    }
}
    


#the h-bond energy proc

proc E-hbond {AA A D H frame} {
    global AEXP REXP HAEX AAEX CUTHB CTOFHB CTONHB CUTHA CTOFHA CTONHA A0 B0

    set aa1 [atomselect top "index $AA" frame $frame]
 #   puts [$aa1 get { x y z }]
    set a  [atomselect top "index $A" frame $frame]
 #   puts [$a get { x y z }]
    set d  [atomselect top "index $D" frame $frame]
 #   puts [$d get { x y z }]
    set h  [atomselect top "index $H" frame $frame]
#    puts [$h get { x y z }]

# atom vectors
    set rAD [vecsub [lindex [$d get { x y z }] 0 ] [lindex [$a get { x y z }] 0 ]]
#    puts $rAD
#    puts [veclength $rAD]
    set rAH [vecsub [lindex [$h get { x y z }] 0 ] [lindex [$a get { x y z }] 0 ]]
    
    set rHD [vecsub [lindex [$d get { x y z }] 0 ] [lindex [$h get { x y z }] 0 ]]
    set rAA_A [vecsub [lindex [$a get { x y z }] 0 ] [lindex [$aa1 get { x y z }] 0 ]]
    set rDH [vecscale -1 $rHD]
    set rHA [vecscale -1 $rAH]
    set rA_AA [vecscale -1 $rAA_A]
    set rDA [vecscale -1 $rAD]

    set CosAHD [vecdot [vecnorm $rHA] [vecnorm $rHD]]
  #  puts $CosAHD
    set CosAA_AH [vecdot [vecnorm $rA_AA] [vecnorm $rAH]]
  #  puts $CosAA_AH
    set CosADH [vecdot [vecnorm $rDA] [vecnorm $rDH]]

# energy function...changed from xplor manual to a "better" angle switching function
     set NRG [expr (($A0/pow([veclength $rAD],$AEXP)) - ($B0/pow([veclength $rAD],$REXP))) * pow($CosAHD,$HAEX) * pow($CosAA_AH,$AAEX) * [SW [expr pow([veclength $rAD],2)] [expr pow($CTONHB,2)] [expr pow($CTOFHB,2)]]   * [SW $CosAHD [expr cos($CTONHA)] [expr cos($CTOFHA)]] ]

 
 #X-plor angle switching part is below...it must be wrong
     #* [SW [expr pow($CosADH,2)] [expr pow([expr cos($CTONHA)],2)] [expr pow([expr cos($CTOFHA)],2)]] ]

    $aa1 delete
    $a delete
    $h delete
    $d delete
    
    unset rAD
    unset rAH
    unset rHD
    unset rAA_A
    unset rDH
    unset rHA
    unset rA_AA
    unset rDA

    return $NRG

    }



# We now calculate the h-bond energy for each of the sets of atoms included above.
# This will be stored in an array that must be initialized. The final value of h-bond 
# energy of an atom will be the sum of the energies of every h-bond it participates in.
# As the acceptor antecedent and acceptor always participate together, and so do the
# donor with its hydrogen, we only need to store the value of the energy  for two of the 
# four atoms.

# Get the energy for each set of atoms, for each timestep!

if {[file exists "$datfile"]} {
    puts "\n$datfile already exists, loading calculated hbond energy..."
    set fil [open $datfile r]

    for {set ts 0} {$ts < $nf} {incr ts} {

	#Initialize the elements to be used in the array
	foreach a $acc d $don {
	    set nrg($ts.$a) 0.
	    set nrg($ts.$d) 0.
	}
    }

    while {[gets $fil dataline] != -1} {
	scan $dataline "%ld %ld %ld %ld %ld %f" time h i j k e
	set nrg($time.$i) [expr $nrg($time.$i) + $e]
	set nrg($time.$j) [expr $nrg($time.$j) + $e]
    }

    # close the data file
    close $fil
} else {

    set fil [open $datfile w]

    for {set ts 0} {$ts < $nf} {incr ts} {

	puts "Calculating hbond energy at frame [expr $ts + 1]..."
	#Initialize the elements to be used in the array
	foreach a $acc d $don {
	    set nrg($ts.$a) 0.
	    set nrg($ts.$d) 0.
	}
	foreach h $aa i $acc j $don k $hyd { 
	    set e [ E-hbond $h $i $j $k $ts]
	    if { [format "%6.3f" $e] != 0 } {
		puts $fil "$ts $h $i $j $k [format "%6.3f" $e]"
	    }
	    set nrg($ts.$i) [expr $nrg($ts.$i) + $e]
	    set nrg($ts.$j) [expr $nrg($ts.$j) + $e]
	    unset e
	}
    }

    # close the data file
    close $fil
}


#Create a lowpass filter to smooth the data
# This one averages over frames defined by $win, starting in the current frame.

proc lowpass {frame win} {

global nrg aa acc don hyd

foreach h $aa i $acc j $don k $hyd { 

    set avgnrg($i) 0.
    set avgnrg($j) 0.

    set ts2 [expr $frame + $win - 1]
    for {set ts $frame} {$ts <= $ts2} {incr ts} {

	set avgnrg($i) [expr $avgnrg($i) + $nrg($ts.$i)]
	set avgnrg($j) [expr $avgnrg($j) + $nrg($ts.$j)]
     }

     set avgnrg($i) [expr $avgnrg($i)/$win]
     set avgnrg($j) [expr $avgnrg($j)/$win]

     set asel [atomselect top "index $h $i" frame $frame]
     $asel set user $avgnrg($i)

     set asel [atomselect top "index $j $k" frame $frame]
     $asel set user $avgnrg($j)

  }
}

# define hydrogen bonds between backbone atoms for displaying them in VMD
proc set-hbond {} {

global aa acc don hyd

foreach h $aa i $acc j $don k $hyd {

    set sel [atomselect top "index $i $k"]
 #   set selnum [$sel num]

    # VMD atomselect returns the index list in ascent order
    # setbonds requires a one-to-one mapping, so be careful.
    if {$i < $k} { 
	$sel setbonds [list [list $h $k] [list $i $j]]
    } else { $sel setbonds [list [list $i $j] [list $h $k]] }

 }
}

set-hbond

for {set ts 0} {$ts < [expr $nf - $lowpassWin]} {incr ts} {
    puts "Processing frame [expr $ts + 1]..."
    lowpass $ts $lowpassWin
}

#Disply all hydrogen bonds, the color indicate the energy level
#for each bond. Green is strong and blue is weak. A bond
#disappears when it is broken.

set nrep [molinfo top get numreps]

color scale method GWB
mol color User
mol representation Bonds 0.4 8.0
mol selection user < -0.1 and name O HN
mol material Opaque
mol addrep top
mol colupdate $nrep top on
mol selupdate $nrep top on
mol scaleminmax top $nrep -4.000000 0.000000
